#pragma once
#include <iostream>
#include <fstream>
#include <algorithm>
#include <iterator>
#include <vector>
#include <deque>
#include <list>
#include <string>
#include <sstream>

void createData_stream() {
	std::srand(std::time(nullptr));
	std::ofstream ofstr;
	ofstr.open("Data.txt");
	for (int i = 0; i < 99; ++i) ofstr << -50 + rand() % 101 << "\t";
	ofstr << -50 + rand() % 101 << std::endl;
	ofstr.close();
}

void createData_generate() {
	std::vector<int> vec(99);
	std::generate(vec.begin(), vec.end(), []() {return -50 + rand() % 101; });
	std::ofstream ofstr;
	ofstr.open("Data.txt");
	std::copy(vec.begin(), vec.end(), std::ostream_iterator<int>(ofstr, "\t"));
	ofstr << *(vec.end() - 1) << std::endl;
	ofstr.close();
}

template<typename _Container>
void loadData_stream(_Container& container, const std::string& name_file) {
	std::ifstream ifstr(name_file, std::ios::out);
	if (!ifstr) {
		std::cerr << "Cannot open file " << name_file << std::endl;
		exit(EXIT_FAILURE);
	}
	std::string line;
	while (std::getline(ifstr, line, '\t')) {
		std::istringstream iss(line);
		typename _Container::value_type elem;
		if (!(iss >> elem)) break;
		container.push_back(elem);
	}
	ifstr.close();
	std::cout << "loadData_stream : " << std::endl;
	for (const auto& e : container) std::cout << e << "  ";
}

template<typename _Container>
void loadData_copy(_Container& t_container, const std::string& name_file) {
	std::ifstream ifstr(name_file, std::ios::out);
	if (!ifstr) {
		std::cerr << "Cannot open file " << name_file << std::endl;
		exit(EXIT_FAILURE);
	}
	else {
		std::istream_iterator<typename _Container::value_type> iterBegin(ifstr);
		std::istream_iterator<typename _Container::value_type> iterEnd;
		std::copy(iterBegin, iterEnd, t_container.begin());
		std::cout << std::endl << "loadData_copy : \n";
		for (const auto& e : t_container) std::cout << e << "  ";
	}
	ifstr.close();
}



template<typename _Container>
_Container modify_1(_Container& container)
{
	using value_type = typename _Container::value_type;
	value_type sum = 0;

	_Container res;
	for (const auto& num : container) {
		sum += num;
	}
	for (auto& num : container) {
		num -= sum;
		res.push_back(num);

	}
	
	return res;

}

template<typename Iter>
void modify_iterator(Iter iter_beg, Iter iter_end) {

	using value_type = typename Iter::value_type;
	value_type sum = 0;	
	for (auto num = iter_beg; num != iter_end; ++num) {
		sum += *num;
	}
	for (auto num = iter_beg; num != iter_end; ++num) {
		*num -= sum;
	}
}

template<typename _Container>
void modify_for_each(_Container& container) {
	using value_type = typename _Container::value_type;
	value_type sum = 0;
	for_each(container.begin(), container.end(), [&sum](const auto& num) {
		sum = +num;
		});
	for_each(container.begin(), container.end(), [sum](auto& num) {
		num -= sum;
		});

}


void modify_transform(const std::string& input_name_file, const std::string& output_name_file) {
	std::ifstream fin(input_name_file);
	std::ofstream fout(output_name_file, std::ios::app);
	if (!fin) std::cerr << "Cannot open file " << input_name_file << std::endl;
	else {
		std::vector<int> vec(100);
		std::copy(std::istream_iterator<int>(fin), std::istream_iterator<int>(), vec.begin());
		int sum = 0;
		for_each(std::istream_iterator<int>(fin), std::istream_iterator<int>(), [&sum](int num) {
			sum += num;
			});
		auto modify = [&](auto& num) {
			return (num -= sum);
		};

		std::transform(vec.begin(), vec.end(),
			std::ostream_iterator<int>(fout, " "),
			modify);

	}
}



template <typename _Container>
void outputResult_stream(const std::string& name_file, const _Container& t_container) {
	std::ofstream ostr(name_file, std::ios::app);
	if (!ostr) {
		std::cerr << "Cannot open file " << name_file << std::endl;
		exit(EXIT_FAILURE);
	}
	for (const auto& e : t_container) ostr << e << "\t";
	ostr << std::endl;
	ostr.close();
}

template <typename _Container>
void outputResult_copy(const std::string& name_file, const _Container& t_container) {
	std::ofstream ostr(name_file, std::ios::app);
	std::copy(t_container.begin(), t_container.end(), std::ostream_iterator<typename _Container::value_type>(ostr, "\t"));
	ostr << std::endl;
	ostr.close();
}



