#include "Record.h"

#include "Record.h"
#include <iostream>
#include <fstream>
#include <iomanip>
#include <algorithm>
#include <ranges>

namespace rv = std::ranges::views;

//----------------------------- Phone -----------------------------
phone::phone(
	const char* p_name,
	const char* p_number,
	const char * p_date,
	double p_discount,
	double p_percent_discount,
	const char* p_time_start,
	const char* p_time_end)
	: discount(p_discount)
	, percent_discount(p_percent_discount)
	
{
	strcpy_s(name, 128, p_name);
	strcpy_s(number, 128, p_number);
	strcpy_s(date, 128, p_date); 
	strcpy_s(time_start, 128, p_time_start);
	strcpy_s(time_end, 128, p_time_end);
}


std::istream& operator>>(std::istream& p_in, phone& p_prod)
{
	p_in.getline(p_prod.name, 128);
	p_in.getline(p_prod.number, 128);
	p_in.getline(p_prod.date, 128);
	p_in >> p_prod.discount;
	p_in >> p_prod.percent_discount;
	p_in.ignore(std::numeric_limits<std::streamsize>::max(), '\n');

	p_in.getline(p_prod.time_start, 128);
	p_in.getline(p_prod.time_end, 128);
	
	
	return p_in;
}

std::ostream& operator<<(std::ostream& p_out, const phone& p_phone)
{
	p_out << p_phone.name << std::endl
		<< p_phone.number << std::endl
		<< p_phone.date << std::endl
		<< p_phone.discount << std::endl
		<< p_phone.percent_discount << std::endl
		<< p_phone.time_start << std::endl
		<< p_phone.time_end << std::endl;

	return p_out;
}

//----------------------------- File -----------------------------

File::File(const std::string& p_fname)
	:m_fname(p_fname)
{}

void File::create_file()
{
	if (first_create)
		std::cout << "File has been already created" << std::endl;
	else {
		std::ofstream fout(m_fname, std::ios::binary);

		int eof;
		phone ele;
		std::cout << "Enter new record" << std::endl;
		std::cout << " |Name | Number | Date | Discount | Discount Percentage | Time start | Time end |" << std::endl;
		do {
			std::cin >> ele;
			fout.write(reinterpret_cast<char*>(&ele), sizeof(phone));
			std::cout << "0 - End" << std::endl
				<< "1 - Continue " << std::endl
				<< ">> ";
			std::cin >> eof;
			if (eof < 0 || eof > 1) {
				std::cout << "Invalid" << std::endl;
				break;
			}
		} while (eof == 1);
	}
}

void File::get_data()
{
	std::ifstream fin(m_fname, std::ios::binary);
	phone ele;
	int idx = 0;
	while (fin.read(reinterpret_cast<char*>(&ele), sizeof(phone))) {
		idx++;
		m_arr.insert(std::make_pair(idx, ele));
	}
}

void File::add_to_file(const std::initializer_list<phone>& p_list)
{
	std::ofstream fout(m_fname, std::ios::binary | std::ios::app);
	for (const auto& ele : p_list)
		fout.write(reinterpret_cast<const char*>(&ele), sizeof(phone));
}

void File::delete_from_file(int p_idx)
{
	get_data();
	m_arr.erase(p_idx);
	std::ofstream fout(m_fname + "_updated", std::ios::binary);
	for (const auto& ele : m_arr)
		fout.write(reinterpret_cast<const char*>(&ele.second), sizeof(phone));
}

void File::replace_in_file(int p_idx, const phone& p_prod)
{
	get_data();
	std::ofstream fout(m_fname + "_updated", std::ios::binary);

	m_arr[p_idx] = p_prod;
	for (const auto& ele : m_arr)
		fout.write(reinterpret_cast<const char*>(&ele.second), sizeof(phone));
}

void File::transform_to_text() {
	get_data();
	std::ofstream fout(m_fname + ".txt");
	fout << "-------------------------------------------------------- Phone record ------------------------------------------------------------------ " << std::endl;
	fout << "| N |      Name      |     Number     |      Date      |    Discount    |   Discount percentage(%)   |   Time start   |    Time end    |" << std::endl;
	size_t count = 0;
	for (const auto& ele : m_arr) {
		count++;
		fout << "|" << std::setw(3) << count << "|"
			<< std::setw(16) << ele.second.name << "|"
			<< std::setw(16) << ele.second.number << "|"
			<< std::setw(16) << ele.second.date << "|"
			<< std::setw(16) << ele.second.discount << "|"
			<< std::setw(28) << ele.second.percent_discount << "|"
			<< std::setw(16) << ele.second.time_start << "|"
			<< std::setw(16) << ele.second.time_end << "|"
			<< std::endl;
	}
}

void File::search_by_date(const std::string& p_date)
{
	get_data();
	std::ofstream fout(p_date, std::ios::binary);

	std::vector<phone> res;

	std::ranges::copy(
		m_arr
		| rv::transform([](const std::pair<int, phone>& ele) {return ele.second; })
		| rv::filter([&p_date](const phone& ele) { return ele.date == p_date; })
		, std::back_inserter(res));

	for (const auto& ele : res)
		fout.write(reinterpret_cast<const char*>(&ele), sizeof(phone));


}

void File::search_by_name(const std::string& p_name)
{
	get_data();
	std::ofstream fout(p_name, std::ios::binary);

	std::vector<phone> res;

	std::ranges::copy(
		m_arr
		| rv::transform([](const std::pair<int, phone>& ele) {return ele.second; })
		| rv::filter([&p_name](const phone& ele) { return ele.name == p_name; })
		, std::back_inserter(res));

	for (const auto& ele : res)
		fout.write(reinterpret_cast<const char*>(&ele), sizeof(phone));


}

void File::search_by_number(const std::string& p_number)
{
	get_data();
	std::ofstream fout(p_number, std::ios::binary);

	std::vector<phone> res;

	std::ranges::copy(
		m_arr
		| rv::transform([](const std::pair<int, phone>& ele) {return ele.second; })
		| rv::filter([&p_number](const phone& ele) { return ele.name == p_number; })
		, std::back_inserter(res));

	for (const auto& ele : res)
		fout.write(reinterpret_cast<const char*>(&ele), sizeof(phone));


}

